# Generated by Django 3.2.20 on 2023-08-18 10:59

import django.contrib.postgres.fields
from django import VERSION
from django.db import migrations, models

if VERSION[0] == 3:
    from django.contrib.postgres.fields.jsonb import JSONField
else:
    from django.db.models import JSONField


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ConfigRow',
            fields=[
                ('name', models.TextField(primary_key=True, serialize=False)),
                ('value', JSONField(blank=True, null=True)),
                ('topic', models.TextField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('tags', django.contrib.postgres.fields.ArrayField(base_field=models.TextField(blank=True, null=True), blank=True, null=True, size=None)),
                ('last_read', models.DateTimeField(blank=True, null=True)),
                ('last_set', models.DateTimeField(blank=True, null=True)),
            ],
        ),
    ]
