# Generated by Django 5.0.4 on 2024-05-01 13:40

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('liveconfigs', '0003_configrow_default_value'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='configrow',
            name='tags',
        ),
    ]
