# Generated by Django 5.0.4 on 2024-05-01 13:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('liveconfigs', '0004_remove_configrow_tags'),
    ]

    operations = [
        migrations.AddField(
            model_name='configrow',
            name='tags',
            field=models.JSONField(blank=True, null=True),
        ),
    ]
